﻿using Kompas6Constants;
using KompasAPI7;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using static System.Net.Mime.MediaTypeNames;

namespace KompasVar
{
    /// <summary>
    /// Логика взаимодействия для MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }
        //----------------------------------------------------------------------------------------------------------------------
        //получить массив из данных неизвестного типа
        public static Array GetArrObject(object inObjects)
        {
            if (inObjects == null)
                return Array.Empty<object>();   //если пусто отдать пустой массив
            if (inObjects is Array array)
                return (Array)inObjects;        //если массив - отдать его
            else
                return new[] { inObjects };     //если одно значение -> в массив его и отдать
        }

        //возьмем первую цилиндрическую грань компонента 
        public static IFace GetCilindricFace(object inArray, Dictionary<string, string> mode = null)
        {
            IFace face = null;
            foreach (IFace item in GetArrObject(inArray)) //IFeature7
            {
                switch (mode["type"])
                {
                    case "Cylinder":
                        if (item.IsCylinder)
                            face = item;
                        break;
                    case "Planar":
                        if (item.IsPlanar)
                        {
                            if (mode["side"] == "up" && item.NormalOrientation)
                                face = item;
                            if (mode["side"] == "down" && !item.NormalOrientation)
                                face = item;
                        }
                        break;
                }
            }
            return face;
        }
        //----------------------------------------------------------------------------------------------------------------------
        //рекурсия по сборке (наьрать массив типа лист)
        public static void getDetails(IPart7 part, List<IPart7> parts)
        {
            parts.Add(part);
            foreach (IPart7 item in part.Parts)
            {
                if (item.Detail)
                    parts.Add(item);
                else
                    getDetails(item, parts);
            }
        }
        //посмотреть на сборку какие детали какие сопряжения (массив ключ значение на словаре)
        public static Dictionary<string, string> GetInfo(IPart7 part7)
        {
            IMateConstraints3D mateConstraints3D = part7.MateConstraints;
            var dic = new Dictionary<string, string>();
            int i = 0;
            List<IPart7> parts = new List<IPart7>();
            getDetails(part7, parts);
            //цикл по деталям
            foreach (IPart7 part in parts)
            {
                dic.Add(i++.ToString("D4") + "  ■ " + (part.Detail ? "ДЕТАЛЬ" : "СБОРКА") + " ", part.Reference.ToString() + " (part.Referense)");
                dic.Add(i++.ToString("D4") + "\t" + part.Marking + "-" + part.Name, "part.Name*************");
                //цикл по поверхностям детали
                foreach (IFace item in GetArrObject(part.Owner.ModelObjects[Kompas6Constants3D.ksObj3dTypeEnum.o3d_face])) //IFeature7
                {
                    if (item.IsPlanar)
                    {
                        dic.Add(i++.ToString("D4") + "   ● ПЛОСКОСТЬ ", item.Reference.ToString() + " (item.Reference)");
                        dic.Add(i++.ToString("D4") + "\t" + item.BaseSurface3DType.ToString(), "item.BaseSurface3DType (ksMathSurface3DTypeEnum)");
                        dic.Add(i++.ToString("D4") + "\t" + item.NormalOrientation.ToString(), "item.NormalOrientation");
                        dic.Add(i++.ToString("D4") + "\t" + item.Surface3DType.ToString(), "item.Surface3DType (ksMathSurface3DTypeEnum)");
                    }
                }
                //цикл по сопряжениям деттали
                foreach (IMateConstraint3D mate in GetArrObject(mateConstraints3D.ObjectConstraints[(IModelObject)part]))
                {
                    if (mate.ConstraintType == Kompas6Constants3D.MateConstraintType.mc_Distance)
                    {
                        dic.Add(i++.ToString("D4") + "   ● СОПРЯЖЕНИЕ ", mate.Reference.ToString() + " (mate.Reference)");
                        dic.Add(i++.ToString("D4") + "\t" + mate.Name, "mate.Name-------------------");
                        dic.Add(i++.ToString("D4") + "\t" + mate.Alignment.ToString(), "mate.Alignment (ksMateConstraintAlignmentEnum)");
                        dic.Add(i++.ToString("D4") + "\t" + mate.BaseObject1.Reference.ToString(), "mate.BaseObject1.Reference");
                        dic.Add(i++.ToString("D4") + "\t" + mate.BaseObject1.Part.Reference.ToString(), "**mate.BaseObject1.Part.Reference");
                        dic.Add(i++.ToString("D4") + "\t" + mate.BaseObject2.Reference.ToString(), "mate.BaseObject2.Reference");
                        dic.Add(i++.ToString("D4") + "\t" + mate.BaseObject2.Part.Reference.ToString(), "**mate.BaseObject2.Part.Reference");
                        dic.Add(i++.ToString("D4") + "\t" + mate.ConstraintType.ToString(), "mate.ConstraintType (MateConstraintType)");
                        dic.Add(i++.ToString("D4") + "\t" + mate.Fixed.ToString(), "mate.Fixed (ksMateFixedTypeEnum)");
                        dic.Add(i++.ToString("D4") + "\t" + mate.ParamValue.ToString(), "mate.ParamValue");

                    }
                }
            }
            return dic;
        }
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            IApplication application = (IApplication)Marshal.GetActiveObject("KOMPAS.Application.7");
            IKompasDocument3D document3D_7 = (IKompasDocument3D)application.ActiveDocument;
            IPart7 part7 = document3D_7.TopPart;
            StringBuilder sb = new StringBuilder();
            foreach (KeyValuePair<string, string> item in GetInfo(part7))
            {
                sb.AppendFormat("{0} - {1}{2}", item.Key, item.Value, Environment.NewLine);
            }
            string result = sb.ToString().TrimEnd();//when converting to string we also want to trim the redundant new line at the very end
                                                    //MessageBox.Show(result);
                                                    // Create a RichTextBlock, a Paragraph and a Run.

            tb1.Text = result;  
        }

        private void btn3_Click(object sender, RoutedEventArgs e)
        {
            IApplication application = (IApplication)Marshal.GetActiveObject("KOMPAS.Application.7");
            IKompasDocument3D document3D_7 = (IKompasDocument3D)application.ActiveDocument;
            IPart7 part7 = document3D_7.TopPart;

            IMateConstraints3D mateConstraints3D = part7.MateConstraints;
            int count = mateConstraints3D.Count;

            IMateConstraint3D mc = mateConstraints3D.MateConstraint3D[0];

            IFeature7 feature7 = part7.Owner;
            object[] sub = feature7.SubFeatures[ksTreeTypeEnum.ksOperTree, true, true];
            object[] arrMc = mateConstraints3D.ObjectConstraints[(ModelObject)feature7];


            tb2.Text = count.ToString();
        }

        private void btn2_Click(object sender, RoutedEventArgs e)
        {
            IApplication application = (IApplication)Marshal.GetActiveObject("KOMPAS.Application.7");
            IKompasDocument3D document3D_7 = (IKompasDocument3D)application.ActiveDocument;
            string fnName7 = document3D_7.PathName;
            IPart7 part7 = document3D_7.TopPart;
            IMateConstraints3D mateConstraints3D = part7.MateConstraints;

            tb2.Text = ((IKompasDocument3D1)document3D_7).MateConstraints.Count.ToString();    //отметимся на форме (в качестве отладки)

            //выбор детали (локалхост) для вставки    
            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.Filter = "Файлы деталей Компас (*.m3d, *.m3d)|*.m3d;*.m3d";
            openFileDialog1.Title = "Выберите файл детали для вставки";
            string insertModel = "";
            if (openFileDialog1.ShowDialog() == true)
                {
                insertModel = openFileDialog1.FileName;
                MessageBox.Show("Зайдите в компас и выберите пару компонентов/деталей сборки для вставки детали между ними", "Вставка между двумя");
            }
            else
            {
                MessageBox.Show("Файл детали для вставки не выбран", "Выбор файла детали");
                return;
            }

            //выбрать на сборке два соседних компонента ВРУЧНУЮ
            ISelectionManager selectionManager = (ISelectionManager)document3D_7.SelectionManager;
            List<IPart7> selectedParts = new List<IPart7>(); //список из парочки объектов
            foreach (IModelObject item in GetArrObject(selectionManager.SelectedObjects))
            {
                if (item.ModelObjectType == Kompas6Constants3D.ksObj3dTypeEnum.o3d_part)    //если это деталь
                    selectedParts.Add((IPart7)item);        //добавим в список
            }
            if (selectedParts.Count != 2)
            {
                MessageBox.Show("Мы же просили выбрать два(две)", "Вставка между двумя");
                return;
            }
            else
            {
                //тут разорвать сопряжения между двумя выделенными объектами/деталями/компонентами
                List<IMateConstraint3D> deleteMateConstraints = new List<IMateConstraint3D>();
                //пройдем по всем сопряжениям первого (индекс 0) объекта и соберем все сопряжения со вторым объектом (индекс 1)
                foreach (IMateConstraint3D item in GetArrObject(mateConstraints3D.ObjectConstraints[(IModelObject)selectedParts[0]]))
                {
                    if (item.BaseObject1.Part.Reference == selectedParts[1].Reference || item.BaseObject1.Part.Reference == selectedParts[0].Reference)
                        deleteMateConstraints.Add(item);    //если объекты совпадают=Ю в коллекцию их
                }
                //удалим все привязки между этими двумя объектами
                if (deleteMateConstraints.Count != 0)
                    ((IKompasDocument1)document3D_7).Delete(deleteMateConstraints.ToArray());

                tb2.Text = ((IKompasDocument3D1)document3D_7).MateConstraints.Count.ToString();    //отметимся на форме (в качестве отладки)

                //вставить в сборку деталь из диалога выбора файла
                IPart7 newPart = part7.Parts.AddFromFile(insertModel, false, true);

                //прикрутить к ней сопряжения между выбранными деталями
                //добавим соосность между выбраннми компонентами
                IMateConstraint3D newMateConstraint3D = mateConstraints3D.Add(Kompas6Constants3D.MateConstraintType.mc_Concentric);
                newMateConstraint3D.BaseObject1 = GetCilindricFace(selectedParts[0].Owner.ModelObjects[Kompas6Constants3D.ksObj3dTypeEnum.o3d_face], new Dictionary<string, string>() { ["type"] = "Cylinder" });
                newMateConstraint3D.BaseObject2 = GetCilindricFace(selectedParts[1].Owner.ModelObjects[Kompas6Constants3D.ksObj3dTypeEnum.o3d_face], new Dictionary<string, string>() { ["type"] = "Cylinder" });
                newMateConstraint3D.Update();

                tb2.Text = ((IKompasDocument3D1)document3D_7).MateConstraints.Count.ToString();    //отметимся на форме (в качестве отладки)

                //добавим соосность между новым и одним из выбных компонентов
                newMateConstraint3D = mateConstraints3D.Add(Kompas6Constants3D.MateConstraintType.mc_Concentric);
                newMateConstraint3D.BaseObject1 = GetCilindricFace(selectedParts[0].Owner.ModelObjects[Kompas6Constants3D.ksObj3dTypeEnum.o3d_face], new Dictionary<string, string>() { ["type"] = "Cylinder" });
                newMateConstraint3D.BaseObject2 = GetCilindricFace(newPart.Owner.ModelObjects[Kompas6Constants3D.ksObj3dTypeEnum.o3d_face], new Dictionary<string, string>() { ["type"] = "Cylinder" });
                newMateConstraint3D.Update();

                tb2.Text = ((IKompasDocument3D1)document3D_7).MateConstraints.Count.ToString();    //отметимся на форме (в качестве отладки)

                //новый объект между старыми
                //дбавим расстояния между новым и первым оъектом
                newMateConstraint3D = mateConstraints3D.Add(Kompas6Constants3D.MateConstraintType.mc_Distance);
                newMateConstraint3D.BaseObject1 = GetCilindricFace(newPart.Owner.ModelObjects[Kompas6Constants3D.ksObj3dTypeEnum.o3d_face], new Dictionary<string, string>() { ["type"] = "Planar", ["side"] = "down" });
                newMateConstraint3D.BaseObject2 = GetCilindricFace(selectedParts[0].Owner.ModelObjects[Kompas6Constants3D.ksObj3dTypeEnum.o3d_face], new Dictionary<string, string>() { ["type"] = "Planar", ["side"] = "down" });
                newMateConstraint3D.ParamValue = 25;
                newMateConstraint3D.Alignment = Kompas6Constants3D.ksMateConstraintAlignmentEnum.ksMCAlignmentOpposite;
                newMateConstraint3D.Update();

                tb2.Text = ((IKompasDocument3D1)document3D_7).MateConstraints.Count.ToString();    //отметимся на форме (в качестве отладки)

                //дбавим расстояния между новым и вторым оъектом
                newMateConstraint3D = mateConstraints3D.Add(Kompas6Constants3D.MateConstraintType.mc_Distance);
                newMateConstraint3D.BaseObject1 = GetCilindricFace(newPart.Owner.ModelObjects[Kompas6Constants3D.ksObj3dTypeEnum.o3d_face], new Dictionary<string, string>() { ["type"] = "Planar", ["side"] = "up" });
                newMateConstraint3D.BaseObject2 = GetCilindricFace(selectedParts[1].Owner.ModelObjects[Kompas6Constants3D.ksObj3dTypeEnum.o3d_face], new Dictionary<string, string>() { ["type"] = "Planar", ["side"] = "up" });
                newMateConstraint3D.ParamValue = 20;// 100;
                newMateConstraint3D.Alignment = Kompas6Constants3D.ksMateConstraintAlignmentEnum.ksMCAlignmentOpposite;
                newMateConstraint3D.Update();

                tb2.Text = ((IKompasDocument3D1)document3D_7).MateConstraints.Count.ToString();    //отметимся на форме (в качестве отладки)
            }
        }
    }
}
